@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,11 +35,11 @@ import json
 import os
 import posixpath
 import tempfile
-from typing import Optional, Union
+from typing import Dict, Optional, Tuple, Union
 
 from absl import logging
 import six
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 
 from tensorflow_datasets.core import file_adapters
 from tensorflow_datasets.core import lazy_imports_lib
@@ -54,37 +54,69 @@ from tensorflow_datasets.core.utils import type_utils
 
 from google.protobuf import json_format
 
+# TODO(b/109648354): Remove the "pytype: disable" comment.
+Nest = Union[Tuple["Nest", ...], Dict[str, "Nest"], str]  # pytype: disable=not-supported-yet
+SupervisedKeysType = Union[Tuple[Nest, Nest], Tuple[Nest, Nest, Nest]]
+
 # Name of the file to output the DatasetInfo protobuf object.
 DATASET_INFO_FILENAME = "dataset_info.json"
 LICENSE_FILENAME = "LICENSE"
 METADATA_FILENAME = "metadata.json"
 
 
+@six.add_metaclass(abc.ABCMeta)
+class Metadata(dict):
+  """Abstract base class for DatasetInfo metadata container.
+
+  `builder.info.metadata` allows the dataset to expose additional general
+  information about the dataset which are not specific to a feature or
+  individual example.
+
+  To implement the interface, overwrite `save_metadata` and
+  `load_metadata`.
+
+  See `tfds.core.MetadataDict` for a simple implementation that acts as a
+  dict that saves data to/from a JSON file.
+  """
+
+  @abc.abstractmethod
+  def save_metadata(self, data_dir):
+    """Save the metadata."""
+    raise NotImplementedError()
+
+  @abc.abstractmethod
+  def load_metadata(self, data_dir):
+    """Restore the metadata."""
+    raise NotImplementedError()
+
+
 class DatasetInfo(object):
   """Information about a dataset.
 
   `DatasetInfo` documents datasets, including its name, version, and features.
   See the constructor arguments and properties for a full list.
 
-  Note: Not all fields are known on construction and may be updated later
-  by `compute_dynamic_properties`. For example: the min and max values of a
-  feature is typically updated during data generation (i.e. on calling
-  builder.download_and_prepare()`).
+  Note: Not all fields are known if the dataset hasn't been generated yet
+  (before the first `builder.download_and_prepare()` call). For example splits
+  names or number of examples might be missing (as they are computed
+  at dataset creation time).
+
   """
 
   def __init__(
       self,
       *,
       builder,
-      description=None,
-      features=None,
-      supervised_keys=None,
+      description: Optional[str] = None,
+      features: Optional[feature_lib.FeatureConnector] = None,
+      supervised_keys: Optional[SupervisedKeysType] = None,
       disable_shuffling: bool = False,
-      homepage=None,
-      citation=None,
-      metadata=None,
+      homepage: Optional[str] = None,
+      citation: Optional[str] = None,
+      metadata: Optional[Metadata] = None,
       license: Optional[str] = None,  # pylint: disable=redefined-builtin
-      redistribution_info=None):
+      redistribution_info: Optional[Dict[str, str]] = None,
+      split_dict: Optional[splits_lib.SplitDict] = None):
     """Constructs DatasetInfo.
 
     Args:
@@ -92,23 +124,39 @@ class DatasetInfo(object):
       description: `str`, description of this dataset.
       features: `tfds.features.FeaturesDict`, Information on the feature dict of
         the `tf.data.Dataset()` object from the `builder.as_dataset()` method.
-      supervised_keys: `tuple` of `(input_key, target_key)`, Specifies the input
-        feature and the label for supervised learning, if applicable for the
-        dataset. The keys correspond to the feature names to select in
-        `info.features`. When calling `tfds.core.DatasetBuilder.as_dataset()`
-        with `as_supervised=True`, the `tf.data.Dataset` object will yield the
-        (input, target) defined here.
+      supervised_keys: Specifies the input structure for supervised learning, if
+        applicable for the dataset, used with "as_supervised". The keys
+        correspond to the feature names to select in `info.features`. When
+        calling `tfds.core.DatasetBuilder.as_dataset()` with
+        `as_supervised=True`, the `tf.data.Dataset` object will yield the
+        structure defined by the keys passed here, instead of that defined by
+        the `features` argument. Typically this is a `(input_key, target_key)`
+        tuple, and the dataset yields a tuple of tensors `(input, target)`
+        tensors.
+
+        To yield a more complex structure, pass a tuple of `tf.nest` compatible
+        structures of feature keys. The resulting `Dataset` will yield
+        structures with each key replaced by the coresponding tensor. For
+        example, passing a triple of keys would return a dataset
+        that yields `(feature, target, sample_weights)` triples for keras.
+        Using `supervised_keys=({'a':'a','b':'b'}, 'c')` would create a dataset
+        yielding a tuple with a dictionary of features in the `features`
+        position.
+
+        Note that selecting features in nested `tfds.features.FeaturesDict`
+        objects is not supported.
       disable_shuffling: `bool`, specify whether to shuffle the examples.
       homepage: `str`, optional, the homepage for this dataset.
       citation: `str`, optional, the citation to use for this dataset.
       metadata: `tfds.core.Metadata`, additonal object which will be
         stored/restored with the dataset. This allows for storing additional
         information with the dataset.
-      license: Optional license of the dataset
-      redistribution_info: `dict`, optional, information needed for
-        redistribution, as specified in `dataset_info_pb2.RedistributionInfo`.
-        The content of the `license` subfield will automatically be written to a
-        LICENSE file stored with the dataset.
+      license: license of the dataset.
+      redistribution_info: information needed for redistribution, as specified
+        in `dataset_info_pb2.RedistributionInfo`. The content of the `license`
+        subfield will automatically be written to a LICENSE file stored with the
+        dataset.
+      split_dict: information about the splits in this dataset.
     """
     self._builder = builder
 
@@ -123,6 +171,7 @@ class DatasetInfo(object):
         name=builder.name,
         description=utils.dedent(description),
         version=str(builder.version),
+        release_notes=builder.release_notes,
         disable_shuffling=disable_shuffling,
         config_name=config_name,
         config_description=config_description,
@@ -142,11 +191,11 @@ class DatasetInfo(object):
             "the top-level. Got {}".format(features))
     self._features = features
     self._splits = splits_lib.SplitDict([], dataset_name=self._builder.name)
+    if split_dict:
+      self.set_splits(split_dict)
     if supervised_keys is not None:
-      assert isinstance(supervised_keys, tuple)
-      assert len(supervised_keys) == 2
-      self._info_proto.supervised_keys.input = supervised_keys[0]
-      self._info_proto.supervised_keys.output = supervised_keys[1]
+      self._info_proto.supervised_keys.CopyFrom(
+          _supervised_keys_to_proto(supervised_keys))
 
     if metadata and not isinstance(metadata, Metadata):
       raise ValueError(
@@ -157,14 +206,42 @@ class DatasetInfo(object):
     # Is this object initialized with both the static and the dynamic data?
     self._fully_initialized = False
 
+  @classmethod
+  def from_proto(cls, builder,
+                 proto: dataset_info_pb2.DatasetInfo) -> "DatasetInfo":
+    """Instantiates DatasetInfo from the given builder and proto."""
+    if builder.builder_config:
+      assert builder.builder_config.name == proto.config_name
+    assert str(builder.version) == proto.version
+    features = None
+    if proto.HasField("features"):
+      features = feature_lib.FeatureConnector.from_proto(proto.features)
+    supervised_keys = None
+    if proto.HasField("supervised_keys"):
+      supervised_keys = _supervised_keys_from_proto(proto.supervised_keys)
+    return cls(
+        builder=builder,
+        description=proto.description,
+        features=features,
+        supervised_keys=supervised_keys,
+        disable_shuffling=proto.disable_shuffling,
+        citation=proto.citation,
+        license=proto.redistribution_info.license,
+        split_dict=splits_lib.SplitDict.from_proto(proto.name, proto.splits),
+    )
+
   @property
-  def as_proto(self):
+  def as_proto(self) -> dataset_info_pb2.DatasetInfo:
     return self._info_proto
 
   @property
-  def name(self):
+  def name(self) -> str:
     return self.as_proto.name
 
+  @property
+  def config_name(self) -> str:
+    return self.as_proto.config_name
+
   @property
   def full_name(self):
     """Full canonical name: (<dataset_name>/<config_name>/<version>)."""
@@ -182,6 +259,10 @@ class DatasetInfo(object):
   def version(self):
     return self._builder.version
 
+  @property
+  def release_notes(self) -> Optional[Dict[str, str]]:
+    return self._builder.release_notes
+
   @property
   def disable_shuffling(self) -> bool:
     return self.as_proto.disable_shuffling
@@ -193,7 +274,7 @@ class DatasetInfo(object):
     return urls and urls[0] or tfds_homepage
 
   @property
-  def citation(self):
+  def citation(self) -> str:
     return self.as_proto.citation
 
   @property
@@ -222,22 +303,22 @@ class DatasetInfo(object):
     return self._features
 
   @property
-  def metadata(self):
+  def metadata(self) -> Optional[Metadata]:
     return self._metadata
 
   @property
-  def supervised_keys(self):
+  def supervised_keys(self) -> Optional[SupervisedKeysType]:
     if not self.as_proto.HasField("supervised_keys"):
       return None
     supervised_keys = self.as_proto.supervised_keys
-    return (supervised_keys.input, supervised_keys.output)
+    return _supervised_keys_from_proto(supervised_keys)
 
   @property
   def redistribution_info(self):
     return self.as_proto.redistribution_info
 
   @property
-  def module_name(self):
+  def module_name(self) -> str:
     return self.as_proto.module_name
 
   @property
@@ -276,14 +357,14 @@ class DatasetInfo(object):
     self.as_proto.file_format = new_file_format.value
 
   @property
-  def splits(self):
+  def splits(self) -> splits_lib.SplitDict:
     return self._splits
 
   def set_splits(self, split_dict: splits_lib.SplitDict) -> None:
     """Split setter (private method)."""
     if self._builder.name != split_dict._dataset_name:  # pylint: disable=protected-access
       raise AssertionError(
-          "SplitDict dataset_name does not seems to match dataset_info. "  # pylint: disable=protected-access
+          "SplitDict dataset_name does not seem to match dataset_info. "  # pylint: disable=protected-access
           f"{self._builder.name} != {split_dict._dataset_name}")
 
     # If the statistics have been pre-loaded, forward the statistics
@@ -309,7 +390,7 @@ class DatasetInfo(object):
       self.as_proto.splits.add().CopyFrom(split_info)
 
   @property
-  def initialized(self):
+  def initialized(self) -> bool:
     """Whether DatasetInfo has been fully initialized."""
     return self._fully_initialized
 
@@ -320,10 +401,10 @@ class DatasetInfo(object):
     return os.path.join(dataset_info_dir, LICENSE_FILENAME)
 
   @property
-  def as_json(self):
+  def as_json(self) -> str:
     return json_format.MessageToJson(self.as_proto, sort_keys=True)
 
-  def write_to_directory(self, dataset_info_dir):
+  def write_to_directory(self, dataset_info_dir) -> None:
     """Write `DatasetInfo` as JSON to `dataset_info_dir`."""
     # Save the features structure & metadata (vocabulary, labels,...)
     if self.features:
@@ -340,7 +421,7 @@ class DatasetInfo(object):
     with tf.io.gfile.GFile(self._dataset_info_path(dataset_info_dir), "w") as f:
       f.write(self.as_json)
 
-  def read_from_directory(self, dataset_info_dir):
+  def read_from_directory(self, dataset_info_dir: str) -> None:
     """Update DatasetInfo from the JSON files in `dataset_info_dir`.
 
     This function updates all the dynamically generated fields (num_examples,
@@ -360,8 +441,8 @@ class DatasetInfo(object):
     json_filename = self._dataset_info_path(dataset_info_dir)
     if not tf.io.gfile.exists(json_filename):
       raise FileNotFoundError(
-          "Try to load `DatasetInfo` from a directory which does not exist or "
-          "does not contain `dataset_info.json`. Please delete the directory "
+          "Tried to load `DatasetInfo` from a directory which does not exist or"
+          " does not contain `dataset_info.json`. Please delete the directory "
           f"`{dataset_info_dir}`  if you are trying to re-generate the "
           "dataset.")
 
@@ -430,7 +511,7 @@ class DatasetInfo(object):
     # Mark as fully initialized.
     self._fully_initialized = True
 
-  def initialize_from_bucket(self):
+  def initialize_from_bucket(self) -> None:
     """Initialize DatasetInfo from GCS bucket info files."""
     # In order to support Colab, we use the HTTP GCS API to access the metadata
     # files. They are copied locally and then loaded.
@@ -484,6 +565,82 @@ class DatasetInfo(object):
     return "\n".join(lines)
 
 
+def _nest_to_proto(nest: Nest) -> dataset_info_pb2.SupervisedKeys.Nest:
+  """Creates a `SupervisedKeys.Nest` from a limited `tf.nest` style structure.
+
+  Args:
+    nest: A `tf.nest` structure of tuples, dictionaries or string feature keys.
+
+  Returns:
+    The same structure as a `SupervisedKeys.Nest` proto.
+  """
+  nest_type = type(nest)
+  proto = dataset_info_pb2.SupervisedKeys.Nest()
+  if nest_type is tuple:
+    for item in nest:
+      proto.tuple.items.append(_nest_to_proto(item))
+  elif nest_type is dict:
+    nest = {key: _nest_to_proto(value) for key, value in nest.items()}
+    proto.dict.CopyFrom(dataset_info_pb2.SupervisedKeys.Dict(dict=nest))
+  elif nest_type is str:
+    proto.feature_key = nest
+  else:
+    raise ValueError("The nested structures in `supervised_keys` must only "
+                     "contain instances of (tuple, dict, str), no subclasses.\n"
+                     f"Found type: {nest_type}")
+
+  return proto
+
+
+def _supervised_keys_to_proto(
+    keys: SupervisedKeysType) -> dataset_info_pb2.SupervisedKeys:
+  """Converts a `supervised_keys` tuple to a SupervisedKeys proto."""
+  if not isinstance(keys, tuple) or len(keys) not in [2, 3]:
+    raise ValueError(
+        "`supervised_keys` must contain a tuple of 2 or 3 elements.\n"
+        f"got: {keys!r}")
+
+  proto = dataset_info_pb2.SupervisedKeys(
+      tuple=dataset_info_pb2.SupervisedKeys.Tuple(
+          items=(_nest_to_proto(key) for key in keys)))
+  return proto
+
+
+def _nest_from_proto(proto: dataset_info_pb2.SupervisedKeys.Nest) -> Nest:
+  """Creates a `tf.nest` style structure from a `SupervisedKeys.Nest` proto.
+
+  Args:
+    proto: A `SupervisedKeys.Nest` proto.
+
+  Returns:
+    The proto converted to a `tf.nest` style structure of tuples, dictionaries
+    or strings.
+  """
+  if proto.HasField("tuple"):
+    return tuple(_nest_from_proto(item) for item in proto.tuple.items)
+  elif proto.HasField("dict"):
+    return {
+        key: _nest_from_proto(value) for key, value in proto.dict.dict.items()
+    }
+  elif proto.HasField("feature_key"):
+    return proto.feature_key
+  else:
+    raise ValueError("`SupervisedKeys.Nest` proto must contain one of "
+                     f"(tuple, dict, feature_key). Got: {proto}")
+
+
+def _supervised_keys_from_proto(
+    proto: dataset_info_pb2.SupervisedKeys) -> SupervisedKeysType:
+  """Converts a `SupervisedKeys` proto back to a simple python tuple."""
+  if proto.input and proto.output:
+    return (proto.input, proto.output)
+  elif proto.tuple:
+    return tuple(_nest_from_proto(item) for item in proto.tuple.items)
+  else:
+    raise ValueError("A `SupervisedKeys` proto must have either `input` and "
+                     "`output` defined, or `tuple`, got: {proto}")
+
+
 def _indent(content):
   """Add indentation to all lines except the first."""
   lines = content.split("\n")
@@ -515,9 +672,12 @@ def get_dataset_feature_statistics(builder, split):
   if filetype_suffix not in ["tfrecord", "csv"]:
     raise ValueError(
         "Cannot generate statistics for filetype {}".format(filetype_suffix))
-  filepattern = naming.filepattern_for_dataset_split(builder.name, split,
-                                                     builder.data_dir,
-                                                     filetype_suffix)
+  filename_template = naming.ShardedFileTemplate(
+      data_dir=builder.data_dir,
+      dataset_name=builder.name,
+      split=split,
+      filetype_suffix=filetype_suffix)
+  filepattern = filename_template.sharded_filepaths_pattern()
   # Avoid generating a large number of buckets in rank histogram
   # (default is 1000).
   stats_options = tfdv.StatsOptions(
@@ -548,6 +708,27 @@ def read_from_json(path: type_utils.PathLike) -> dataset_info_pb2.DatasetInfo:
   return parsed_proto
 
 
+def read_proto_from_builder_dir(
+    builder_dir: type_utils.PathLike) -> dataset_info_pb2.DatasetInfo:
+  """Reads the dataset info from the given builder dir.
+
+  Args:
+    builder_dir: The folder that contains the dataset info files.
+
+  Returns:
+    The DatasetInfo proto as read from the builder dir.
+
+  Raises:
+    FileNotFoundError: If the builder_dir does not exists.
+  """
+  info_path = os.path.join(
+      os.path.expanduser(builder_dir), DATASET_INFO_FILENAME)
+  if not tf.io.gfile.exists(info_path):
+    raise FileNotFoundError(
+        f"Could not load dataset info: {info_path} does not exists.")
+  return read_from_json(info_path)
+
+
 def pack_as_supervised_ds(
     ds: tf.data.Dataset,
     ds_info: DatasetInfo,
@@ -562,32 +743,6 @@ def pack_as_supervised_ds(
     return ds
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Metadata(dict):
-  """Abstract base class for DatasetInfo metadata container.
-
-  `builder.info.metadata` allows the dataset to expose additional general
-  information about the dataset which are not specific to a feature or
-  individual example.
-
-  To implement the interface, overwrite `save_metadata` and
-  `load_metadata`.
-
-  See `tfds.core.MetadataDict` for a simple implementation that acts as a
-  dict that saves data to/from a JSON file.
-  """
-
-  @abc.abstractmethod
-  def save_metadata(self, data_dir):
-    """Save the metadata."""
-    raise NotImplementedError()
-
-  @abc.abstractmethod
-  def load_metadata(self, data_dir):
-    """Restore the metadata."""
-    raise NotImplementedError()
-
-
 def _metadata_filepath(data_dir):
   return os.path.join(data_dir, METADATA_FILENAME)
 
