@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,17 +22,17 @@ import tempfile
 import numpy as np
 import pytest
 
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 from tensorflow_datasets import testing
 from tensorflow_datasets.core import dataset_info
 from tensorflow_datasets.core import features
 from tensorflow_datasets.core import file_adapters
 from tensorflow_datasets.core import read_only_builder
 from tensorflow_datasets.core import utils
+from tensorflow_datasets.core.proto import dataset_info_pb2
+from tensorflow_datasets.core.proto import feature_pb2
 from tensorflow_datasets.image_classification import mnist
 
-tf.enable_v2_behavior()
-
 _TFDS_DIR = utils.tfds_path()
 _INFO_DIR = os.path.join(_TFDS_DIR, "testing", "test_data", "dataset_info",
                          "mnist", "3.0.1")
@@ -113,6 +113,12 @@ class DatasetInfoTest(testing.TestCase):
     self.assertEqual(info.module_name, "tensorflow_datasets.testing.test_utils")
     self.assertEqual(False, info.disable_shuffling)
 
+    self.assertEqual(info.version, utils.Version("1.0.0"))
+    self.assertEqual(info.release_notes, {
+        "1.0.0": "Release notes 1.0.0",
+        "2.0.0": "Release notes 2.0.0"
+    })
+
   def test_disable_shuffling(self):
     info = dataset_info.DatasetInfo(
         builder=self._builder, disable_shuffling=True)
@@ -218,6 +224,11 @@ class DatasetInfoTest(testing.TestCase):
       # Even though restored_info has been restored, informations defined in
       # the code overwrite informations from the json file.
       self.assertEqual(restored_info.description, "A description")
+      self.assertEqual(restored_info.version, utils.Version("1.0.0"))
+      self.assertEqual(restored_info.release_notes, {
+          "1.0.0": "Release notes 1.0.0",
+          "2.0.0": "Release notes 2.0.0"
+      })
       self.assertEqual(restored_info.supervised_keys,
                        ("input (new)", "output (new)"))
       self.assertEqual(restored_info.homepage, "http://some-location-new")
@@ -341,6 +352,30 @@ def test_file_format_values(tmp_path: pathlib.Path):
     testing.DummyDataset(data_dir=tmp_path, file_format="arrow")
 
 
+def test_dataset_info_from_proto():
+  builder = RandomShapedImageGenerator(data_dir=testing.make_tmp_dir())
+  train = dataset_info_pb2.SplitInfo(
+      name="train", num_shards=2, shard_lengths=[4, 5])
+  test = dataset_info_pb2.SplitInfo(
+      name="test", num_shards=3, shard_lengths=[1, 2, 3])
+  text_feature = feature_pb2.Feature(
+      python_class_name="tensorflow_datasets.core.features.text_feature.Text",
+      text=feature_pb2.TextFeature())
+  proto = dataset_info_pb2.DatasetInfo(
+      name="random_shaped_image_generator",
+      version=str(builder.version),
+      features=feature_pb2.Feature(
+          python_class_name="tensorflow_datasets.core.features.features_dict.FeaturesDict",
+          features_dict=feature_pb2.FeaturesDict(
+              features={"text": text_feature})),
+      splits=[train, test])
+  result = dataset_info.DatasetInfo.from_proto(builder=builder, proto=proto)
+  assert result.splits["test"].shard_lengths == test.shard_lengths
+  assert result.splits["train"].shard_lengths == train.shard_lengths
+  assert set(result.features.keys()) == {"text"}
+  assert result.version == builder.version
+
+
 # pylint: disable=g-inconsistent-quotes
 _INFO_STR = '''tfds.core.DatasetInfo(
     name='mnist',
