@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,15 +16,15 @@
 """Download manager interface."""
 
 import concurrent.futures
+import dataclasses
 import hashlib
 import typing
 from typing import Any, Dict, Iterator, Optional, Tuple, Union
 import uuid
 
 from absl import logging
-import dataclasses
 import promise
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.download import checksums
@@ -212,7 +212,7 @@ class DownloadManager(object):
 
     self._download_dir: ReadWritePath = download_dir
     self._extract_dir: ReadWritePath = extract_dir
-    self._manual_dir: Optional[ReadOnlyPath] = manual_dir
+    self._manual_dir: Optional[ReadOnlyPath] = manual_dir  # pytype: disable=annotation-type-mismatch  # attribute-variable-annotations
     self._manual_dir_instructions = utils.dedent(manual_dir_instructions)
     self._download_dir.mkdir(parents=True, exist_ok=True)
     self._extract_dir.mkdir(parents=True, exist_ok=True)
@@ -733,7 +733,7 @@ def _validate_checksums(
     if not expected_url_info:
       raise ValueError(f'Missing checksums url: {url}, yet '
                        '`force_checksums_validation=True`. '
-                       'Did you forgot to register checksums ?')
+                       'Did you forget to register checksums?')
 
   if (expected_url_info and computed_url_info and
       expected_url_info != computed_url_info):
@@ -752,7 +752,7 @@ def _read_url_info(url_path: type_utils.PathLike) -> checksums.UrlInfo:
   file_info = resource_lib.read_info_file(url_path)
   if 'url_info' not in file_info:
     raise ValueError(
-        'Could not found `url_info` in {}. This likelly indicates that '
+        'Could not find `url_info` in {}. This likely indicates that '
         'the files where downloaded with a previous version of TFDS (<=3.1.0). '
     )
   url_info = file_info['url_info']
@@ -765,5 +765,4 @@ def _map_promise(map_fn, all_inputs):
   """Map the function into each element and resolve the promise."""
   all_promises = tf.nest.map_structure(map_fn, all_inputs)  # Apply the function
   res = tf.nest.map_structure(lambda p: p.get(), all_promises)  # Wait promises
-
   return res
