@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,10 +17,11 @@
 import functools
 from typing import Any, Dict, Iterator, Union
 
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 
 from tensorflow_datasets.core.features import feature as feature_lib
 from tensorflow_datasets.core.features import sequence_feature
+from tensorflow_datasets.core.utils import py_utils
 from tensorflow_datasets.core.utils import type_utils
 
 
@@ -90,12 +91,14 @@ class Dataset(sequence_feature.Sequence):
 
   # TODO(tfds): Add support for TF1 graph mode.
 
+  @py_utils.memoize()
   def get_tensor_info(self):
     """Shape of one element of the dataset."""
     # Add the dataset level
     tensor_info = self._feature.get_tensor_info()
     return tf.nest.map_structure(_add_dataset_lvl, tensor_info)
 
+  @py_utils.memoize()
   def get_serialized_info(self):
     # Add the dataset level and the number of elements in the dataset
     tensor_info = super().get_serialized_info()
