@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@ import functools
 import io
 import itertools
 import logging
+import operator
 import os
 import random
 import shutil
@@ -29,13 +30,12 @@ import sys
 import textwrap
 import threading
 import typing
-from typing import Any, Callable, Iterator, List, NoReturn, Optional, Tuple, Type, TypeVar, Union
+from typing import Any, Callable, Dict, Iterable, Iterator, List, NoReturn, Optional, Tuple, Type, TypeVar, Union
 import uuid
 
 from six.moves import urllib
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 from tensorflow_datasets.core import constants
-from tensorflow_datasets.core import file_adapters
 from tensorflow_datasets.core.utils import type_utils
 
 Tree = type_utils.Tree
@@ -52,7 +52,7 @@ T = TypeVar('T')
 Fn = TypeVar('Fn', bound=Callable[..., Any])
 
 
-def is_notebook():
+def is_notebook() -> bool:
   """Returns True if running in a notebook (Colab, Jupyter) environment."""
   # Inspired from the tqdm autonotebook code
   try:
@@ -66,6 +66,13 @@ def is_notebook():
     return True
 
 
+def warning(text: str) -> None:
+  if is_notebook():
+    print(text)
+  else:
+    logging.warning(text)
+
+
 @contextlib.contextmanager
 def temporary_assignment(obj, attr, value):
   """Temporarily assign obj.attr to value."""
@@ -99,9 +106,8 @@ def disable_logging():
 class NonMutableDict(dict):
   """Dict where keys can only be added but not modified.
 
-  Will raise an error if the user try to overwrite one key. The error message
-  can be customized during construction. It will be formatted using {key} for
-  the overwritten key.
+  Raises an error if a key is overwritten. The error message can be customized
+  during construction. It will be formatted using {key} for the overwritten key.
   """
 
   def __init__(self, *args, **kwargs):
@@ -202,15 +208,14 @@ def zip_nested(arg0, *args, **kwargs):
   return (arg0,) + args
 
 
-def flatten_nest_dict(d):
+def flatten_nest_dict(d: type_utils.TreeDict[T]) -> Dict[str, T]:
   """Return the dict with all nested keys flattened joined with '/'."""
   # Use NonMutableDict to ensure there is no collision between features keys
   flat_dict = NonMutableDict()
   for k, v in d.items():
     if isinstance(v, dict):
-      flat_dict.update({
-          '{}/{}'.format(k, k2): v2 for k2, v2 in flatten_nest_dict(v).items()
-      })
+      flat_dict.update(
+          {f'{k}/{k2}': v2 for k2, v2 in flatten_nest_dict(v).items()})
     else:
       flat_dict[k] = v
   return flat_dict
@@ -269,7 +274,7 @@ def pack_as_nest_dict(flat_d, nest_d):
       sub_d = {
           k2: flat_d.pop('{}/{}'.format(k, k2)) for k2, _ in v_flat.items()
       }
-      # Recursivelly pack the dictionary
+      # Recursively pack the dictionary
       nest_out_d[k] = pack_as_nest_dict(sub_d, v)
     else:
       nest_out_d[k] = flat_d.pop(k)
@@ -473,6 +478,7 @@ def basename_from_url(url: str) -> str:
 
 def list_info_files(dir_path: type_utils.PathLike) -> List[str]:
   """Returns name of info files within dir_path."""
+  from tensorflow_datasets.core import file_adapters  # pylint: disable=g-import-not-at-top  # pytype: disable=import-error
   path = os.fspath(dir_path)
   return [
       fname for fname in tf.io.gfile.listdir(path)
@@ -501,3 +507,8 @@ def add_sys_path(path: type_utils.PathLike) -> Iterator[None]:
     yield
   finally:
     sys.path.remove(path)
+
+
+def prod(iterable: Iterable[int], *, start=1) -> int:
+  """Backport of python 3.8 `math.prod`."""
+  return functools.reduce(operator.mul, iterable, start)
