@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,7 @@ import pathlib
 
 import pytest
 
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 
 from tensorflow_datasets.testing import test_case
 from tensorflow_datasets.testing import test_utils
@@ -63,6 +63,10 @@ class RunInGraphAndEagerTest(test_case.TestCase):
         super(ExampleTest, self).setUp()
         modes.append('setup_' + mode_name())
 
+      def subTest(self, msg, **params):
+        modes.append('subtest_' + msg)
+        return super().subTest(msg, **params)
+
       @test_utils.run_in_graph_and_eager_modes
       def testBody(self):
         modes.append('run_' + mode_name())
@@ -71,13 +75,19 @@ class RunInGraphAndEagerTest(test_case.TestCase):
     e.setUp()
     e.testBody()
 
-    self.assertEqual(modes[0:2], ['setup_eager', 'run_eager'])
-    self.assertEqual(modes[2:], ['setup_graph', 'run_graph'])
+    self.assertEqual(modes, [
+        'setup_eager',
+        'subtest_eager_mode',
+        'run_eager',
+        'subtest_graph_mode',
+        'setup_graph',
+        'run_graph',
+    ])
 
   def test_mock_tf(self):
     # pylint: disable=g-import-not-at-top,reimported
     import tensorflow as tf_lib1
-    import tensorflow.compat.v2 as tf_lib2
+    import tensorflow as tf_lib2
 
     # pylint: enable=g-import-not-at-top,reimported
 
